local core = LibStub("AceAddon-3.0"):GetAddon("SilverDragon")
local module = core:NewModule("Scan_Vignettes", "AceEvent-3.0", "AceConsole-3.0")
local Debug = core.Debug

local globaldb
function module:OnInitialize()
	globaldb = core.db.global

	self.db = core.db:RegisterNamespace("Scan_Vignettes", {
		profile = {
			enabled = true,
			location = false,
		},
	})

	local config = core:GetModule("Config", true)
	if config then
		config.options.args.scanning.plugins.vignettes = {
			vignettes = {
				type = "group",
				name = "Vignettes",
				get = function(info) return self.db.profile[info[#info]] end,
				set = function(info, v) self.db.profile[info[#info]] = v end,
				args = {
					enabled = config.toggle("Enabled", "Scan minimap vignettes (it's what Blizzard calls them, okay?)", 10),
					location = config.toggle("Record location on vignette appearance", "Record the mob's location when the vignette triggers for it. If this isn't set, it'll wait until you target it and are within interaction range to store the location.", 30),
				},
			},
		}
	end
end

function module:OnEnable()
	self:RegisterEvent("VIGNETTE_ADDED")
end

local already_notified = {}
function module:VIGNETTE_ADDED(event, instanceid, mysterious_number)
	if not instanceid then
		-- ...just in case
		core:Print("No Vignette instanceid")
		return
	end
	if already_notified[instanceid] then
		Debug("Skipping notify", "already done", id)
		return
	end
	already_notified[instanceid] = true
	local x, y, name, iconid = C_Vignettes.GetVignetteInfoFromInstanceID(instanceid)
	-- iconid seems to be 40:chests, 41:mobs
	if not name then
		core:Print("Failed to pull Vignette Name do to blizzard bug with instanceid " ..instanceid)
		PlaySoundFile("Sound\\interface\\UI_RaidBossWhisperWarning.ogg", "Master")
		return
	end
	local mob_id = globaldb.mob_id[name] or globaldb.mob_vignettes[name]
	if mob_id then
		-- it's a rare that we know about!
		-- note, we could instead try using just iconid==41, but I don't know if that's going to actually be all rares yet
		self:NotifyIfNeeded(mob_id)
	end
	if iconid == 40 then
		core:Print("Chest Nearby: "..name)
		PlaySoundFile("Sound\\Doodad\\PortcullisActive_Closed.ogg", "Master")
	end
end

--Maybe special handling with these:
--1. Disabling savemob even if vignette saving is enabled
--2. Ignoring comment from mob data and instead displaying "click to find out more" or something instead of falsely guessing a color
--3. Supress sending sync until targetted so we don't send bad mobId to guild/group either.
--4. Supress alert sound when clicking target to avoid double sound alert that sometimes happen when the vignette triggers for example "slate" but target scan triggers "amber" Instead, the "click to find out more" will fill in with correct comment
local mobsSharingName = { --Mobs that can trigger false alerts using vignettes
	[69769] = true,--Zandalari Warbringer (Slate)
	[69841] = true,--Zandalari Warbringer (Amber)
	[69842] = true,--Zandalari Warbringer (Jade)
	[73174] = true,--Archiereus of Flame (Spawn)
	[73666] = true,--Archiereus of Flame (Summon)
	[51401] = true,--Madexx (Red)
	[51402] = true,--Madexx (Green)
	[51403] = true,--Madexx (Black)
	[51404] = true,--Madexx (Blue)
	[50154] = true,--Madexx (Brown)
}

function module:NotifyIfNeeded(id, instanceid)
	local current_zone, x, y = core:GetPlayerLocation()
	local newloc = false
	if self.db.profile.location and not globaldb.mob_tameable[id] then
		--Pull some info from global database since it's not sent from syncs, and we don't want
		-- to erase that info with savemob function just copy it over.
		local creature_type = globaldb.mob_type[id]
		local elite = globaldb.mob_elite[id]
		local level = globaldb.mob_level[id]
		local name = globaldb.mob_name[id]
		newloc = core:SaveMob(id, name, current_zone, x, y, level, elite, creature_type)
	end
	core:NotifyMob(id, globaldb.mob_name[id], current_zone, x, y, false, newloc, "vignette", false, nil, true)
end
